"use strict";

var _index = _interopRequireDefault(require("../index"));

var _casual = _interopRequireDefault(require("casual"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

_casual.default.seed(10);

var modernHats = [{
  name: "Baseball Cap",
  sorting: {
    popularity: 95
  }
}, {
  name: "Beanie",
  sorting: {
    popularity: 70
  }
}, {
  name: "Golf",
  sorting: {
    popularity: 20
  }
}, {
  name: "Other",
  sorting: {
    popularity: 10
  }
}];
var oldHats = [{
  name: "Top Hat",
  sorting: {
    popularity: 60
  }
}, {
  name: "Beret",
  sorting: {
    popularity: 15
  }
}, {
  name: "Bowler Cap",
  sorting: {
    popularity: 9
  }
}, {
  name: "Sombrero",
  sorting: {
    popularity: 5
  }
}, {
  name: "Stetson",
  sorting: {
    popularity: 2
  }
}];

var getData = function getData(data, page) {
  var pageSize = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 3;
  return data.slice(page * pageSize, (page + 1) * pageSize);
};

describe("combine-paginators", function () {
  var combinedGetters;
  beforeEach(function () {
    combinedGetters = (0, _index.default)({
      getters: [function (page) {
        return getData(modernHats, page);
      }, function (page) {
        return getData(oldHats, page);
      }],
      sortKey: "sorting.popularity"
    });
  });
  describe("test data", function () {
    it("is valid test data, generating out of order results", function () {
      expect([].concat(_toConsumableArray([].concat(_toConsumableArray(getData(modernHats, 0)), _toConsumableArray(getData(oldHats, 0))).sort(function (a, b) {
        return b.sorting.popularity - a.sorting.popularity;
      })), _toConsumableArray([].concat(_toConsumableArray(getData(modernHats, 1)), _toConsumableArray(getData(oldHats, 1))).sort(function (a, b) {
        return b.sorting.popularity - a.sorting.popularity;
      })))).toEqual([modernHats[0], modernHats[1], oldHats[0], modernHats[2], oldHats[1], oldHats[2], modernHats[3], oldHats[3], oldHats[4]]);
    });
  });
  describe("getNext", function () {
    it("get intersecting hits for first page of known data set",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee() {
      var page;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return combinedGetters.getNext();

            case 2:
              page = _context.sent;
              expect(page).toEqual([modernHats[0], modernHats[1], oldHats[0], modernHats[2]]);

            case 4:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    })));
    it("get intersecting hits for second page of known data set",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2() {
      var page;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context2.next = 4;
              return combinedGetters.getNext();

            case 4:
              page = _context2.sent;
              expect(page).toEqual([oldHats[1], modernHats[3]]);

            case 6:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    })));
    it("get hits for third page of known data set",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee3() {
      var page;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context3.next = 4;
              return combinedGetters.getNext();

            case 4:
              _context3.next = 6;
              return combinedGetters.getNext();

            case 6:
              page = _context3.sent;
              expect(page).toEqual([oldHats[2]]);

            case 8:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    })));
    it("get trailing hits for fourth page of known data set",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee4() {
      var page;
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context4.next = 4;
              return combinedGetters.getNext();

            case 4:
              _context4.next = 6;
              return combinedGetters.getNext();

            case 6:
              _context4.next = 8;
              return combinedGetters.getNext();

            case 8:
              page = _context4.sent;
              expect(page).toEqual([oldHats[3], oldHats[4]]);

            case 10:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4);
    })));
    it("return empty array when known data set is exhausted",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee5() {
      var page;
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context5.next = 4;
              return combinedGetters.getNext();

            case 4:
              _context5.next = 6;
              return combinedGetters.getNext();

            case 6:
              _context5.next = 8;
              return combinedGetters.getNext();

            case 8:
              _context5.next = 10;
              return combinedGetters.getNext();

            case 10:
              page = _context5.sent;
              expect(page).toEqual([]);

            case 12:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    }))); // Randomly generating data sets ensures robustness against edge cases

    it("should return all results in order for 1000 random data sets",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee6() {
      var minimumLength, maximumLength, minimumPopularity, maximumPopularity, minimumPageSize, maximumPageSize, minimumNumberOfDataSets, maximumNumberOfDataSets, _loop, index;

      return regeneratorRuntime.wrap(function _callee6$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              minimumLength = 1;
              maximumLength = 500;
              minimumPopularity = 0;
              maximumPopularity = 1000;
              minimumPageSize = 1;
              maximumPageSize = 100;
              minimumNumberOfDataSets = 2;
              maximumNumberOfDataSets = 5;
              _loop =
              /*#__PURE__*/
              regeneratorRuntime.mark(function _loop(index) {
                var maximumPopularityForRun, numberDataSets, dataSets, pageSizeForDataSets, expectedResult, getters, combined, lastResult, allResults;
                return regeneratorRuntime.wrap(function _loop$(_context6) {
                  while (1) {
                    switch (_context6.prev = _context6.next) {
                      case 0:
                        // We ensure same maximum popularity is kept throughout run
                        // This causes data to get distributed
                        maximumPopularityForRun = _casual.default.integer(minimumPopularity, maximumPopularity);
                        numberDataSets = _casual.default.integer(minimumNumberOfDataSets, maximumNumberOfDataSets);
                        dataSets = Array.from({
                          length: numberDataSets
                        }, function () {
                          return Array.from({
                            length: _casual.default.integer(minimumLength, maximumLength)
                          }, function () {
                            return {
                              sorting: {
                                popularity: _casual.default.integer(minimumPopularity, maximumPopularityForRun)
                              }
                            };
                          }).sort(function (a, b) {
                            return b.sorting.popularity - a.sorting.popularity;
                          });
                        });
                        pageSizeForDataSets = Array.from({
                          length: numberDataSets
                        }, function () {
                          return _casual.default.integer(minimumPageSize, maximumPageSize);
                        });
                        expectedResult = dataSets.reduce(function (acc, dataSet) {
                          return [].concat(_toConsumableArray(acc), _toConsumableArray(dataSet));
                        }, []).sort(function (a, b) {
                          return b.sorting.popularity - a.sorting.popularity;
                        });
                        getters = dataSets.map(function (dataSet, index) {
                          return function (page) {
                            return getData(dataSet, page, pageSizeForDataSets[index]);
                          };
                        });
                        combined = (0, _index.default)({
                          getters: getters,
                          sortKey: "sorting.popularity"
                        });
                        lastResult = void 0;
                        allResults = [];

                      case 9:
                        if (!(lastResult !== [])) {
                          _context6.next = 18;
                          break;
                        }

                        _context6.next = 12;
                        return combined.getNext();

                      case 12:
                        lastResult = _context6.sent;

                        if (!(lastResult.length === 0)) {
                          _context6.next = 15;
                          break;
                        }

                        return _context6.abrupt("break", 18);

                      case 15:
                        allResults = [].concat(_toConsumableArray(allResults), _toConsumableArray(lastResult));
                        _context6.next = 9;
                        break;

                      case 18:
                        expect(allResults).toEqual(expectedResult);

                      case 19:
                      case "end":
                        return _context6.stop();
                    }
                  }
                }, _loop);
              });
              index = 0;

            case 10:
              if (!(index < 1000)) {
                _context7.next = 15;
                break;
              }

              return _context7.delegateYield(_loop(index), "t0", 12);

            case 12:
              index++;
              _context7.next = 10;
              break;

            case 15:
            case "end":
              return _context7.stop();
          }
        }
      }, _callee6);
    })));
  });
  describe("getNextForGetter", function () {
    it("should get next results for a specific getter",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee7() {
      var page;
      return regeneratorRuntime.wrap(function _callee7$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.next = 2;
              return combinedGetters.getNextForGetter(0);

            case 2:
              page = _context8.sent;
              expect(page).toEqual([modernHats[0], modernHats[1], modernHats[2]]);

            case 4:
            case "end":
              return _context8.stop();
          }
        }
      }, _callee7);
    })));
    it("should get next results from a specific getter, regardless of the getNext state",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee8() {
      var page;
      return regeneratorRuntime.wrap(function _callee8$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context9.next = 4;
              return combinedGetters.getNextForGetter(0);

            case 4:
              page = _context9.sent;
              expect(page).toEqual([modernHats[0], modernHats[1], modernHats[2]]);

            case 6:
            case "end":
              return _context9.stop();
          }
        }
      }, _callee8);
    })));
    it("should get results for a specific getter, without interfering with getNext",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee9() {
      var page;
      return regeneratorRuntime.wrap(function _callee9$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return combinedGetters.getNext();

            case 2:
              _context10.next = 4;
              return combinedGetters.getNextForGetter(1);

            case 4:
              _context10.next = 6;
              return combinedGetters.getNextForGetter(1);

            case 6:
              _context10.next = 8;
              return combinedGetters.getNext();

            case 8:
              page = _context10.sent;
              expect(page).toEqual([oldHats[1], modernHats[3]]);

            case 10:
            case "end":
              return _context10.stop();
          }
        }
      }, _callee9);
    })));
    it("should return [] when next page is null",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee10() {
      var page;
      return regeneratorRuntime.wrap(function _callee10$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              _context11.next = 2;
              return combinedGetters.injectState({
                getNextForGetter: {
                  nextPageForGetters: [null, null]
                }
              });

            case 2:
              _context11.next = 4;
              return combinedGetters.getNextForGetter(1);

            case 4:
              page = _context11.sent;
              expect(page).toEqual([]);

            case 6:
            case "end":
              return _context11.stop();
          }
        }
      }, _callee10);
    })));
    it("should return [] when pages are exhausted",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee11() {
      var page;
      return regeneratorRuntime.wrap(function _callee11$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _context12.next = 2;
              return combinedGetters.getNextForGetter(1);

            case 2:
              _context12.next = 4;
              return combinedGetters.getNextForGetter(1);

            case 4:
              _context12.next = 6;
              return combinedGetters.getNextForGetter(1);

            case 6:
              page = _context12.sent;
              expect(page).toEqual([]);

            case 8:
            case "end":
              return _context12.stop();
          }
        }
      }, _callee11);
    })));
  });
});