"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.sortAlgolia = void 0;

require("@babel/polyfill");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var get = function get(obj, accessor) {
  return accessor.split(".").reduce(function (o, i) {
    return o[i];
  }, obj);
};
/*
 * sortAlgolia(a: object, b: object, includeGeo: bool)
 *
 * Custom sort method for algolia searches using the default algolia search relevance
 **/


var sortAlgolia = function sortAlgolia(a, b) {
  var includeGeo = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var rankings = [];
  rankings.push({
    name: "nbTypos",
    goal: "lowest"
  }); // Only compare geo  if not remote

  if (includeGeo) {
    rankings.push({
      name: "geoDistance",
      goal: "lowest"
    });
    rankings.push({
      name: "geoPrecision",
      goal: "highest"
    });
  }

  rankings.push({
    name: "firstMatchedWord",
    goal: "lowest"
  });
  rankings.push({
    name: "words",
    goal: "highest"
  });
  rankings.push({
    name: "filters",
    goal: "highest"
  });
  rankings.push({
    name: "proximityDistance",
    goal: "lowest"
  });
  rankings.push({
    name: "nbExactWords",
    goal: "highest"
  });
  rankings.push({
    name: "userScore",
    goal: "highest"
  });

  for (var index = 0; index < rankings.length; index++) {
    var _rankings$index = rankings[index],
        name = _rankings$index.name,
        goal = _rankings$index.goal;
    var score = goal === "lowest" ? a._rankingInfo[name] - b._rankingInfo[name] : b._rankingInfo[name] - a._rankingInfo[name];

    if (score !== 0) {
      return score;
    }
  }

  return 0;
};
/*
 * combinePagination()
 * */


exports.sortAlgolia = sortAlgolia;

var _default = function _default(_ref) {
  var getters = _ref.getters,
      sortKey = _ref.sortKey,
      sort = _ref.sort,
      _ref$sortDirection = _ref.sortDirection,
      sortDirection = _ref$sortDirection === void 0 ? "desc" : _ref$sortDirection;
  var state = {
    pages: getters.map(function () {
      return [];
    }),
    getNext: {
      data: getters.map(function () {
        return [];
      }),
      meta: {},
      nextPageForGetters: getters.map(function () {
        return 0;
      })
    },
    getNextForGetter: {
      nextPageForGetters: getters.map(function () {
        return 0;
      })
    }
  };

  var _getSortKey = function _getSortKey(hit) {
    return get(hit, sortKey);
  };

  var _isAfter = function _isAfter(a, b) {
    var _ref2 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref2$eq = _ref2.eq,
        eq = _ref2$eq === void 0 ? true : _ref2$eq;

    if (sort) {
      return eq ? sort(a, b) >= 0 : sort(a, b) > 0;
    }

    if (sortDirection === "asc") {
      return eq ? _getSortKey(a) >= _getSortKey(b) : _getSortKey(a) > _getSortKey(b);
    }

    return eq ? _getSortKey(a) <= _getSortKey(b) : _getSortKey(a) < _getSortKey(b);
  };

  var _isBefore = function _isBefore(a, b) {
    var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref3$eq = _ref3.eq,
        eq = _ref3$eq === void 0 ? true : _ref3$eq;

    if (sort) {
      return eq ? sort(a, b) <= 0 : sort(a, b) < 0;
    }

    if (sortDirection === "asc") {
      return eq ? _getSortKey(a) <= _getSortKey(b) : _getSortKey(a) < _getSortKey(b);
    }

    return eq ? _getSortKey(a) >= _getSortKey(b) : _getSortKey(a) > _getSortKey(b);
  };

  var _sortPage = function _sortPage(hits) {
    return sort ? hits.sort(sort) : hits.sort(function (a, b) {
      return get(b, sortKey) - get(a, sortKey);
    });
  };

  var _mergeData = function _mergeData(data) {
    return _sortPage(data.reduce(function (acc, hitsForGetter) {
      return [].concat(_toConsumableArray(acc), _toConsumableArray(hitsForGetter));
    }, []));
  };

  var _trimPage = function _trimPage(_ref4) {
    var page = _ref4.page,
        meta = _ref4.meta;
    var earliestLastHit = meta.earliestLastHit,
        firstHit = meta.firstHit; // const lastHitForShortestPage = shortestPage[shortestPage.length - 1];

    if (!_isAfter(earliestLastHit, firstHit)) {
      throw new Error("combined-pagination Error. Order of results from data getters does not match sortKey and sortDirection.");
    }

    var trimmedPage = page.filter(function (hit) {
      return _isAfter(hit, firstHit) && _isBefore(hit, earliestLastHit);
    });
    return trimmedPage;
  };

  var _getMeta = function _getMeta(_ref5) {
    var currentMeta = _ref5.currentMeta,
        results = _ref5.results;
    var lastHitForGetter = results[results.length - 1];
    var _currentMeta$earliest = currentMeta.earliestLastHit,
        earliestLastHit = _currentMeta$earliest === void 0 ? lastHitForGetter : _currentMeta$earliest,
        _currentMeta$firstHit = currentMeta.firstHit,
        firstHit = _currentMeta$firstHit === void 0 ? results[0] : _currentMeta$firstHit;
    return {
      firstHit: _isBefore(results[0], firstHit, {
        eq: false
      }) ? results[0] : firstHit,
      earliestLastHit: _isBefore(lastHitForGetter, earliestLastHit, {
        eq: false
      }) ? lastHitForGetter : earliestLastHit
    };
  };

  var _shouldProcessPage = function _shouldProcessPage(_ref6) {
    var data = _ref6.data,
        page = _ref6.page,
        getterIndex = _ref6.getterIndex;
    return data[getterIndex].length === 0 && page !== null;
  }; // NB Not hugely efficient


  var _tidyData = function _tidyData(_ref7) {
    var data = _ref7.data,
        trimmedPage = _ref7.trimmedPage;
    return data.reduce(function (acc, getterData) {
      return [].concat(_toConsumableArray(acc), [getterData.filter(function (hit) {
        return trimmedPage.indexOf(hit) === -1;
      })]);
    }, []);
  };

  var _getData =
  /*#__PURE__*/
  function () {
    var _ref9 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(_ref8) {
      var getterIndex, page, userOptions, getter, cachedPage, results;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              getterIndex = _ref8.getterIndex, page = _ref8.page, userOptions = _ref8.userOptions;
              getter = getters[getterIndex];
              cachedPage = state.pages[getterIndex][page];

              if (!cachedPage) {
                _context.next = 7;
                break;
              }

              results = cachedPage;
              _context.next = 11;
              break;

            case 7:
              _context.next = 9;
              return getter(page, userOptions);

            case 9:
              results = _context.sent;
              state.pages[getterIndex].push(results);

            case 11:
              return _context.abrupt("return", results);

            case 12:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function _getData(_x) {
      return _ref9.apply(this, arguments);
    };
  }();

  var getNext =
  /*#__PURE__*/
  function () {
    var _ref10 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2(userOptions) {
      var getterIndex, _page, results, page, trimmedPage;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              // We recalculate these meta params on each page
              state.getNext.meta.firstHit = undefined;
              state.getNext.meta.earliestLastHit = undefined;
              getterIndex = 0;

            case 3:
              if (!(getterIndex < getters.length)) {
                _context2.next = 16;
                break;
              }

              _page = state.getNext.nextPageForGetters[getterIndex];

              if (!_shouldProcessPage({
                data: state.getNext.data,
                page: _page,
                getterIndex: getterIndex
              })) {
                _context2.next = 12;
                break;
              }

              _context2.next = 8;
              return _getData({
                getterIndex: getterIndex,
                page: _page,
                userOptions: userOptions
              });

            case 8:
              results = _context2.sent;

              if (results.length > 0) {
                state.getNext.data[getterIndex] = results;
                state.getNext.nextPageForGetters[getterIndex] = _page + 1;
                state.getNext.meta = _getMeta({
                  currentMeta: state.getNext.meta,
                  getterIndex: getterIndex,
                  results: results
                });
              } else {
                state.getNext.nextPageForGetters[getterIndex] = null;
              }

              _context2.next = 13;
              break;

            case 12:
              if (state.getNext.data[getterIndex].length > 0) {
                state.getNext.meta = _getMeta({
                  currentMeta: state.getNext.meta,
                  getterIndex: getterIndex,
                  results: state.getNext.data[getterIndex]
                });
              }

            case 13:
              getterIndex++;
              _context2.next = 3;
              break;

            case 16:
              page = _mergeData(state.getNext.data);
              trimmedPage = page.length > 0 ? _trimPage({
                page: page,
                meta: state.getNext.meta
              }) : page;
              state.getNext.data = _tidyData({
                data: state.getNext.data,
                trimmedPage: trimmedPage
              });
              return _context2.abrupt("return", trimmedPage);

            case 20:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function getNext(_x2) {
      return _ref10.apply(this, arguments);
    };
  }();

  var getNextForGetter =
  /*#__PURE__*/
  function () {
    var _ref11 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee3(getterIndex, userOptions) {
      var page, results;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              page = state.getNextForGetter.nextPageForGetters[getterIndex];

              if (!(page === null)) {
                _context3.next = 3;
                break;
              }

              return _context3.abrupt("return", []);

            case 3:
              _context3.next = 5;
              return _getData({
                getterIndex: getterIndex,
                page: page,
                userOptions: userOptions
              });

            case 5:
              results = _context3.sent;

              if (results.length === 0) {
                state.getNextForGetter.nextPageForGetters[getterIndex] = null;
              } else {
                state.getNextForGetter.nextPageForGetters[getterIndex] = page + 1;
              }

              return _context3.abrupt("return", results);

            case 8:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    return function getNextForGetter(_x3, _x4) {
      return _ref11.apply(this, arguments);
    };
  }();

  var extractState = function extractState() {
    return state;
  };

  var injectState =
  /*#__PURE__*/
  function () {
    var _ref12 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee4(injectedState) {
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              state = injectedState;

            case 1:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4);
    }));

    return function injectState(_x5) {
      return _ref12.apply(this, arguments);
    };
  }();

  return {
    getNext: getNext,
    getNextForGetter: getNextForGetter,
    extractState: extractState,
    injectState: injectState
  };
};

exports.default = _default;